package com.displaynone.acss.controllers.acs;

import com.displaynone.acss.components.acs.ACSComponent;
import com.displaynone.acss.components.acs.models.gate.GateDTO;
import com.displaynone.acss.components.acs.models.gate.GateMapper;
import com.displaynone.acss.components.acs.models.gate.GateModel;
import com.displaynone.acss.components.acs.models.visit.VisitDTO;
import com.displaynone.acss.components.acs.models.visit.VisitMapper;
import com.displaynone.acss.components.auth.AuthComponent;
import com.displaynone.acss.components.auth.models.user.UserModel;
import com.displaynone.acss.exception.generics.ForbiddenHTTPException;
import com.displaynone.acss.exception.generics.NotFoundHTTPException;
import com.displaynone.acss.utils.GlobalUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;


@RestController
@RequestMapping("/api/acs")
@RequiredArgsConstructor
public class ACSController {
    private final ACSComponent acsComponent;
    private final AuthComponent authComponent;

    @PostMapping("/open")
    public ResponseEntity<Void> open(@RequestBody OpenRQB body) {
        Long code = body.getCode();
        GateModel gate = acsComponent.getGateByCodeStrict(code);
        if (gate.isACSBlocked()) throw new ForbiddenHTTPException("Specified gate is blocked");

        UserModel user = (UserModel) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        if (user.isACSBlocked()) throw new ForbiddenHTTPException("User is blocked");

        acsComponent.createVisit(user.getId(), gate.getId());

        return ResponseEntity.ok(null);
    }

    @GetMapping("visits/me")
    public ResponseEntity<Page<VisitDTO>> getMyVisitsPaginated(
            @RequestParam(defaultValue = "0") int page,
            @RequestParam(defaultValue = "10") int size
    ) {
        Pageable pageable = PageRequest.of(page, size);
        UserModel user = (UserModel) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        return ResponseEntity.ok(acsComponent.getVisitsByUserIdPaginated(user.getId(), pageable).map(VisitMapper::convertToDTO));
    }

    @GetMapping("visits/login/{login}")
    public ResponseEntity<Page<VisitDTO>> getUserVisitsPaginated(
            @RequestParam(defaultValue = "0") int page,
            @RequestParam(defaultValue = "10") int size,
            @PathVariable String login
    ) {
        Pageable pageable = PageRequest.of(page, size);
        UserModel user = authComponent.getUserByLoginStrict(login);
        return ResponseEntity.ok(acsComponent.getVisitsByUserIdPaginated(user.getId(), pageable).map(VisitMapper::convertToDTO));
    }

    @GetMapping("visits/gate/{gateId}")
    public ResponseEntity<Page<VisitDTO>> getUserVisitsPaginated(
            @RequestParam(defaultValue = "0") int page,
            @RequestParam(defaultValue = "10") int size,
            @PathVariable Long gateId
    ) {
        Pageable pageable = PageRequest.of(page, size);
        if (!acsComponent.isGateExists(gateId)) throw new NotFoundHTTPException("Gate not found");
        return ResponseEntity.ok(acsComponent.getVisitsByGateIdPaginated(gateId, pageable).map(VisitMapper::convertToDTO));
    }

    @GetMapping("gates/gate/{gateId}")
    public ResponseEntity<GateDTO> getGateById(@PathVariable Long gateId) {
        return ResponseEntity.ok(GateMapper.convertToDTO(acsComponent.getGateByIdStrict(gateId)));
    }

    @PatchMapping("gates/gate/{gateId}")
    public ResponseEntity<Void> updateGateById(@PathVariable Long gateId, @RequestBody GateUpdateRQB body) {
        GateModel gateUpdate = new GateModel();

        GlobalUtils.updateIfNotNull(body.getCode(), gateUpdate::setCode);
        GlobalUtils.updateIfNotNull(body.getActive(), gateUpdate::setActive);
        GlobalUtils.updateIfNotNull(body.getBlocked(), gateUpdate::setACSBlocked);
        acsComponent.updateGate(gateId, gateUpdate);

        return ResponseEntity.ok(null);
    }
}
